import * as i0 from '@angular/core';
import { InjectionToken, EventEmitter, inject, DestroyRef, Input, Output, Directive, NgModule } from '@angular/core';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import { BehaviorSubject, Subscription, isObservable, of, Subject } from 'rxjs';
import { isBlank, isStringMap, isType, isJsObject, isFunction } from '@fundamental-ngx/cdk/utils';
import { map, takeUntil } from 'rxjs/operators';

/**
 * Checks whether passed value is a valid data source.
 */
function isDataSource(value) {
    return value && typeof value.unsubscribe === 'function' && value.dataChanges;
}

/**
 * Injection token used for passing custom datasource transformer into DataSourceProvider.
 */
const FD_DATA_SOURCE_TRANSFORMER = new InjectionToken('FdDataSourceTransformer');

class DataSourceDirective {
    constructor() {
        /**
         * Emits when the data source object has been changed.
         */
        this.dataSourceChanged = new EventEmitter();
        /**
         * Event emitted when datasource content has been changed.
         */
        this.dataChanged = new EventEmitter();
        /**
         * Event emitted when data provider loading state has been changed.
         */
        this.isLoading = new EventEmitter();
        /**
         * Data stream. Emits when new data retrieved.
         */
        this.dataChanged$ = new BehaviorSubject([]);
        /** @hidden */
        this._destroyRef = inject(DestroyRef);
        /** @hidden */
        this._dsSubscription = new Subscription();
        /** @hidden */
        this._dataSourceTransformer = inject(FD_DATA_SOURCE_TRANSFORMER);
    }
    /**
     * Data source.
     * @param source
     */
    set dataSource(source) {
        this._dataSource = source;
        this._initializeDataSource();
        this.dataSourceChanged.next();
    }
    get dataSource() {
        return this._dataSource;
    }
    /** @hidden */
    ngOnDestroy() {
        this.dataSourceProvider?.unsubscribe();
        this._dsSubscription?.unsubscribe();
    }
    /** @Hidden */
    _toDataStream(source) {
        return !source
            ? undefined
            : this._dataSourceTransformer
                ? this._dataSourceTransformer.parse(source)
                : undefined;
    }
    /** @hidden */
    _initializeDataSource() {
        if (isDataSource(this.dataSource)) {
            this.dataSourceProvider?.unsubscribe();
            this._dsSubscription?.unsubscribe();
        }
        // Convert whatever comes in as DataSource, so we can work with it identically
        this.dataSourceProvider = this._toDataStream(this.dataSource);
        if (!this.dataSourceProvider) {
            return;
        }
        this._dsSubscription = new Subscription();
        this._dsSubscription.add(this.dataSourceProvider.dataLoading
            .pipe(takeUntilDestroyed(this._destroyRef))
            .subscribe((isLoading) => this.isLoading.emit(isLoading)));
        this._dsSubscription.add(this.dataSourceProvider.dataChanges.pipe(takeUntilDestroyed(this._destroyRef)).subscribe((data) => {
            this.dataChanged.emit(data);
            this.dataChanged$.next(data);
        }));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DataSourceDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.3", type: DataSourceDirective, isStandalone: true, selector: "[fdkDataSource]", inputs: { dataSource: "dataSource" }, outputs: { dataSourceChanged: "dataSourceChanged", dataChanged: "dataChanged", isLoading: "isLoading" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DataSourceDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdkDataSource]',
                    standalone: true
                }]
        }], propDecorators: { dataSourceChanged: [{
                type: Output
            }], dataChanged: [{
                type: Output
            }], isLoading: [{
                type: Output
            }], dataSource: [{
                type: Input
            }] } });

class DataSourceModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DataSourceModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.3", ngImport: i0, type: DataSourceModule, imports: [DataSourceDirective], exports: [DataSourceDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DataSourceModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: DataSourceModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [DataSourceDirective],
                    exports: [DataSourceDirective]
                }]
        }] });

/** @Hidden */
function getMatchingStrategyStartsWithPerTermReqexp(value) {
    return new RegExp(`(\\s|^)(${value})`, 'gi');
}
/** @hidden */
function objectValues(obj) {
    return Object.keys(obj).map((key) => obj[key]);
}
/** @hidden */
function objectToName(target) {
    if (isBlank(target) || (!isStringMap(target) && !isType(target))) {
        throw new Error(' Cannot convert. Uknown object');
    }
    return isType(target) ? target.prototype.constructor.name : target.constructor.name;
}

var MatchingStrategy;
(function (MatchingStrategy) {
    MatchingStrategy["STARTS_WITH_PER_TERM"] = "starts with per term";
    MatchingStrategy["STARTS_WITH"] = "starts with";
    MatchingStrategy["CONTAINS"] = "contains";
})(MatchingStrategy || (MatchingStrategy = {}));

/**
 * Base data provider class.
 */
class AbstractDataProvider {
    constructor() {
        /** @hidden */
        this._matchingStrategy = MatchingStrategy.STARTS_WITH;
        /** @hidden */
        this._matchingBy = null;
    }
    /** @hidden */
    setLookupKey(key) {
        this._keyPath = key;
    }
    /** @hidden */
    setMatchingBy(matchingBy) {
        this._matchingBy = matchingBy;
    }
    /** @hidden */
    setMatchingStrategy(strategy) {
        this._matchingStrategy = strategy;
    }
}
/**
 * Default data provider class used for other components to extend from.
 */
class DataProvider extends AbstractDataProvider {
    /** @hidden */
    constructor(values) {
        super();
        this.values = values;
        /** Default limit for the pagination. */
        this.defaultLimit = 50;
    }
    /** @hidden */
    fetch(params) {
        const observable = isObservable(this.values) ? this.values : of(this.values);
        const queryString = params.get('query');
        const limit = params.get('limit') || this.defaultLimit;
        if (!queryString || queryString === '*') {
            return observable.pipe(map((items) => this.withLimit(items, limit)));
        }
        const toLowerPattern = queryString.toLowerCase();
        return observable.pipe(map((items) => {
            const result = [];
            for (let i = 0; i < items.length; i++) {
                const item = items[i];
                if (this.matches(item, toLowerPattern)) {
                    result.push(item);
                    if (result.length >= limit) {
                        break;
                    }
                }
            }
            return result;
        }));
    }
    /**
     * Returns observable with the number of total items that satisfy the search query.
     * Warning: this is a naive implementation of the number of the filtered items.
     * For real-world scenarios, it is recommended to override this method with its own logic.
     */
    getTotalItems(params) {
        const observable = isObservable(this.values) ? this.values : of(this.values);
        const queryString = params.get('query');
        if (!queryString) {
            return observable.pipe(map((items) => items.length));
        }
        const toLowerPattern = queryString.toLowerCase();
        return observable.pipe(map((items) => items.filter((item) => this.matches(item, toLowerPattern)).length));
    }
    /**
     * Warning: If you don't supply search Key and you want fulltext search, and you use this
     * default implementation, be aware that it can perform poorly as it is a naive implementation
     * that does not do deep compare.
     */
    matches(item, pattern) {
        if (!this._matchingBy) {
            return false;
        }
        const { firstBy, secondaryBy } = this._matchingBy;
        let matched = this.matchesBy(item, pattern, firstBy);
        if (!matched && secondaryBy) {
            matched = this.matchesBy(item, pattern, secondaryBy);
        }
        return matched;
    }
    /** @hidden */
    matchesBy(item, pattern, matchingBy) {
        const value = isJsObject(item) && matchingBy ? matchingBy(item) : item;
        if (isFunction(value)) {
            return value.call(item);
        }
        else if (isJsObject(value)) {
            return this.hasObjectValue(item, pattern);
        }
        else if (this._matchingStrategy === MatchingStrategy.STARTS_WITH_PER_TERM) {
            const reqexp = getMatchingStrategyStartsWithPerTermReqexp(pattern);
            return !!pattern && !!value && !!value.match(reqexp);
        }
        else if (this._matchingStrategy === MatchingStrategy.STARTS_WITH) {
            return !!pattern && !!value && value.toString().toLowerCase().startsWith(pattern.toLowerCase());
        }
        else if (this._matchingStrategy === MatchingStrategy.CONTAINS) {
            return !!pattern && !!value && value.toString().toLowerCase().indexOf(pattern) > -1;
        }
        else {
            return !!pattern && !!value && value.toString().toLowerCase() === pattern;
        }
    }
    /** @hidden */
    hasObjectValue(obj, pattern) {
        const values = objectValues(obj);
        const parentObj = objectToName(obj);
        const length2 = values.filter((value) => {
            if (!value || Array.isArray(value)) {
                return false;
            }
            else if (!isJsObject(value) && !isFunction(value)) {
                return value.toString().toLowerCase().indexOf(pattern) !== -1;
            }
            else if (isJsObject(value) && objectToName(value) !== parentObj) {
                return this.hasObjectValue(value, pattern);
            }
            return false;
        }).length;
        return length2 > 0;
    }
    /** @hidden */
    withLimit(data, limit) {
        if (limit && data.length > limit) {
            return data.slice(0, limit);
        }
        return data;
    }
}

class BaseDataSource {
    /**
     * Max limit of items to be returned.
     */
    static { this.MaxLimit = 5; }
    /**
     * Emitted when new data has been requested.
     * @returns Observable
     */
    get dataRequested() {
        return this._dataRequested$.asObservable();
    }
    /**
     * Emitted when new data has been received.
     * @returns Observable
     */
    get dataReceived() {
        return this._dataReceived$.asObservable();
    }
    /**
     * Emitted when loading state has been changed.
     * @returns Observable.
     */
    get dataLoading() {
        return this._dataLoading$.asObservable();
    }
    /**
     * Emits when the data from the provider has been changed.
     * @returns Observable of data source objects.
     */
    get dataChanges() {
        return this._dataChanges$.asObservable().pipe(takeUntil(this._destroy$));
    }
    /** @hidden */
    constructor(dataProvider) {
        this.dataProvider = dataProvider;
        /**
         * Whether to enable limit of max items.
         */
        this.limitless = false;
        /** @hidden */
        this._dataChanges$ = new BehaviorSubject([]);
        /** @hidden */
        this._dataRequested$ = new BehaviorSubject(false);
        /** @hidden */
        this._dataReceived$ = new BehaviorSubject(false);
        /** @hidden */
        this._destroy$ = new Subject();
        /** @hidden */
        this._dataLoading$ = new BehaviorSubject(false);
    }
    /**
     * Searches through the data source with defined parameters.
     * @param predicate Search query.
     * @param start start index.
     * @param end end index.
     */
    match(predicate = new Map(), start = 0, end = Infinity) {
        this._dataRequested$.next(true);
        this._dataLoading$.next(true);
        const searchParam = this._getSearchParams(predicate);
        this.dataProvider
            .fetch(searchParam, start, end)
            .pipe(takeUntil(this._destroy$))
            .subscribe({
            next: (result) => {
                this._dataReceived$.next(true);
                this._dataLoading$.next(false);
                this._dataChanges$.next(result);
            },
            error: () => {
                this._dataReceived$.next(false);
                this._dataLoading$.next(false);
            }
        });
    }
    /**
     * Returns the Observable of a total items number filtered by the `predicate` param.
     * @param predicate Search query.
     */
    getTotalItems(predicate = new Map()) {
        const searchParam = this._getSearchParams(predicate);
        return this.dataProvider.getTotalItems(searchParam);
    }
    /**
     * Closes the stream
     */
    unsubscribe() {
        this._destroy$.next();
        this._destroy$.complete();
    }
    /** @hidden */
    _getSearchParams(predicate) {
        const searchParam = new Map();
        if (typeof predicate === 'string') {
            searchParam.set('query', predicate);
        }
        else if (predicate instanceof Map) {
            predicate.forEach((v, k) => searchParam.set(k, v));
        }
        else {
            throw new Error('DataSource.match() predicate can only accepts string and Map');
        }
        if (!searchParam.has('limit') && !this.limitless) {
            searchParam.set('limit', BaseDataSource.MaxLimit);
        }
        return searchParam;
    }
}

const applyDatasourceDirective = {
    directive: DataSourceDirective,
    inputs: ['dataSource'],
    outputs: ['dataChanged']
};

/**
 * Generated bundle index. Do not edit.
 */

export { AbstractDataProvider, BaseDataSource, DataProvider, DataSourceDirective, DataSourceModule, FD_DATA_SOURCE_TRANSFORMER, MatchingStrategy, applyDatasourceDirective, isDataSource };
//# sourceMappingURL=fundamental-ngx-cdk-data-source.mjs.map
