import * as i0 from '@angular/core';
import { InjectionToken, isDevMode, EventEmitter, inject, computed, ElementRef, signal, Input, ViewChild, Output, Directive, NgModule, ChangeDetectorRef, DestroyRef, Injectable } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NgControl, ControlContainer, NgForm } from '@angular/forms';
import { Subject, Subscription } from 'rxjs';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';

const formStates = ['success', 'error', 'warning', 'default', 'information'];

const stateSet = new Set(formStates);
/** Checks if the given state is a valid control state. */
function isValidControlState(value) {
    return stateSet.has(value);
}

const FD_FORM_FIELD_CONTROL = new InjectionToken('FdFormFieldControl');

const FD_FORM_FIELD = new InjectionToken('FdFormField');

let randomId = 0;
class CvaDirective {
    /**
     *  The state of the form control - applies css classes.
     *  Can be 'success', 'error', 'warning', 'default', 'information'.
     *
     * @default 'default'
     */
    set state(state) {
        if (!state || isValidControlState(state)) {
            this._state$.set(state);
        }
        else if (isDevMode()) {
            console.warn(`Provided value "${state}" is not a valid option for FormStates type`);
        }
    }
    get state() {
        return this.normalizedState$();
    }
    /** Whether the input is disabled */
    set disabled(value) {
        this.setDisabledState(value);
    }
    get disabled() {
        if (this.ngControl && this.ngControl.disabled !== null) {
            return this.ngControl.disabled;
        }
        return this._disabled;
    }
    /**
     * Tell the component if we are in editing mode.
     */
    set editable(value) {
        const newVal = coerceBooleanProperty(value);
        if (this._editable !== newVal) {
            this._editable = newVal;
            this._markForCheck();
            this.stateChanges.next('editable');
        }
    }
    get editable() {
        return this._editable;
    }
    /** Whether control has errors */
    get controlInvalid() {
        return this._controlInvalid$();
    }
    /**
     * Element reference.
     */
    get elementRef() {
        // Return a fallback ElementRef if _elementRefOrNull is null
        return this._elementRefOrNull || this._elementRef;
    }
    /** @hidden */
    constructor() {
        /**
         * Emits when change detection is needed.
         */
        this.detectChanges = new EventEmitter();
        /**
         * Emits when mark for changes detection is needed.
         */
        this.markForCheck = new EventEmitter();
        /**
         * See @FormFieldControl
         */
        this.focused = false;
        /**
         * See @FormFieldControl
         */
        this.stateChanges = new Subject();
        /** @hidden */
        this.formField = inject(FD_FORM_FIELD, {
            skipSelf: true,
            optional: true
        });
        /**
         * NgControl instance.
         */
        this.ngControl = inject(NgControl, {
            optional: true,
            host: true
        });
        /**
         * Form container instance. Usually ngForm or FormGroup directives.
         */
        this.controlContainer = inject(ControlContainer, {
            optional: true,
            skipSelf: true
        });
        /**
         * Separate NgForm instance. For cases when formGroup is used with the form itself.
         */
        this.ngForm = inject(NgForm, {
            optional: true,
            skipSelf: true
        });
        /** @hidden */
        this.normalizedState$ = computed(() => {
            const storedState = this._state$();
            if (storedState) {
                return storedState;
            }
            if (!this._controlInvalid$()) {
                return 'default';
            }
            return this.formField?.getPriorityState() || 'default';
        });
        /** @hidden */
        this._subscriptions = new Subscription();
        /**
         * A private property to hold the ElementRef which might be null
         */
        this._elementRefOrNull = inject(ElementRef, { optional: true });
        /** @hidden */
        this._defaultId = `fd-input-id-${randomId++}`;
        /** ID for the Element */
        // eslint-disable-next-line @typescript-eslint/member-ordering
        this.id = this._defaultId;
        /** @hidden */
        this._editable = true;
        /**
         * @hidden
         */
        this._controlInvalid$ = signal(false);
        /** @hidden */
        this._parentControl = inject(FD_FORM_FIELD_CONTROL, {
            skipSelf: true,
            optional: true
        });
        /**
         * @hidden
         * The state of the form control - applies css classes.
         * Can be `success`, `error`, `warning`, `information` or 'default'
         */
        this._state$ = signal(null);
        /** @hidden */
        this.onChange = () => { };
        /** @hidden */
        this.onTouched = () => { };
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /** @hidden */
    ngOnInit() {
        // There may be cases when one form control is used as a base to build another form control.
        if (!this._parentControl) {
            this.formField?.registerFormFieldControl(this);
        }
    }
    /**
     * Re-validate and emit event to parent container on every CD cycle as they are some errors
     * that we can't subscribe to.
     */
    ngDoCheck() {
        if (this.ngControl) {
            this.updateErrorState();
        }
    }
    /** @hidden */
    ngAfterViewInit() {
        if (this.ngControl) {
            this._subscriptions.add(this.ngControl.statusChanges?.subscribe(() => {
                this._markForCheck();
            }));
        }
        const labelAndHelpId = `fdp-form-label-content-${this.id}`;
        // if not specified, associate label and inline help ids with the input,
        // else add these ids to the specified ones
        if (!this.ariaLabelledBy) {
            this.ariaLabelledBy = labelAndHelpId;
        }
        else {
            this.ariaLabelledBy += ' ' + labelAndHelpId;
        }
        this._markForCheck();
    }
    /** @hidden */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /** @hidden */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /** @hidden */
    ngOnDestroy() {
        this._subscriptions.unsubscribe();
        this.stateChanges.complete();
        this.formField?.unregisterFormFieldControl(this);
    }
    /** @hidden */
    setDisabledState(isDisabled) {
        const newState = coerceBooleanProperty(isDisabled);
        this._markForCheck();
        if (newState !== this._disabled) {
            this._disabled = newState;
            this.stateChanges.next('setDisabledState');
        }
    }
    /**
     * Method for setting the value
     * @param value
     */
    writeValue(value) {
        this.value = value;
        this.stateChanges.next('writeValue');
        this._markForCheck();
    }
    /**
     *
     * Keeps track of element focus
     */
    _onFocusChanged(isFocused) {
        if (isFocused !== this.focused && (!this.disabled || !isFocused)) {
            this.focused = isFocused;
            this.stateChanges.next('_onFocusChanged');
        }
        if (!isFocused) {
            this.onTouched();
        }
    }
    /**
     * Handles even when we click on parent container which is the FormField Wrapping this
     * control
     */
    onContainerClick(event) {
        this.focus(event);
    }
    /**
     * In most of the cases when working with input element directly you should be just find to assign
     * variable to this element
     *
     * ```
     * <input #inputElementRef fd-form-control ...>
     * ```
     *
     * and this default behavior used. For other cases implement focus.
     *
     */
    focus(event) {
        if (this._elementRef && !this.focused) {
            this._elementRef.nativeElement.focus(event);
        }
    }
    /**
     *  Need re-validates errors on every CD iteration to make sure we are also
     *  covering non-control errors, errors that happens outside of this control
     */
    updateErrorState() {
        const parent = this.ngForm;
        const parentControlContainer = this.controlContainer;
        const control = this.ngControl ? this.ngControl.control : null;
        const newStatusIsError = !!(control?.invalid &&
            (control.dirty || control.touched || parent?.submitted || parentControlContainer?.submitted));
        if (newStatusIsError !== this._controlInvalid$()) {
            this._controlInvalid$.set(newStatusIsError);
            this.stateChanges.next('updateErrorState');
            this._markForCheck();
        }
    }
    /**
     * Used to change the value of a control.
     * @param value the value to be applied
     * @param emitOnChange whether to emit "onChange" event.
     * Should be "false", if the change is made programmatically (internally) by the control, "true" otherwise
     */
    setValue(value, emitOnChange = true) {
        if (value !== this.value) {
            this.writeValue(value);
            if (emitOnChange) {
                this.onChange(value);
            }
            this._markForCheck();
        }
    }
    /** @hidden */
    _markForCheck() {
        this.markForCheck.emit();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: CvaDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.3", type: CvaDirective, isStandalone: true, selector: "[fdkCva]", inputs: { placeholder: "placeholder", state: "state", stateMessage: "stateMessage", disabled: "disabled", readonly: "readonly", ariaLabelledBy: "ariaLabelledBy", ariaLabel: "ariaLabel", editable: "editable", name: "name", id: "id" }, outputs: { detectChanges: "detectChanges", markForCheck: "markForCheck" }, viewQueries: [{ propertyName: "_elementRef", first: true, predicate: ["inputElementRef"], descendants: true, read: ElementRef, static: true }], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: CvaDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[fdkCva]',
                    standalone: true
                }]
        }], ctorParameters: () => [], propDecorators: { placeholder: [{
                type: Input
            }], state: [{
                type: Input
            }], stateMessage: [{
                type: Input
            }], disabled: [{
                type: Input
            }], readonly: [{
                type: Input
            }], ariaLabelledBy: [{
                type: Input
            }], ariaLabel: [{
                type: Input
            }], editable: [{
                type: Input
            }], name: [{
                type: Input
            }], detectChanges: [{
                type: Output
            }], markForCheck: [{
                type: Output
            }], _elementRef: [{
                type: ViewChild,
                args: ['inputElementRef', { static: true, read: ElementRef }]
            }], id: [{
                type: Input
            }] } });

class FormsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: FormsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.3", ngImport: i0, type: FormsModule, imports: [CvaDirective], exports: [CvaDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: FormsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: FormsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CvaDirective],
                    exports: [CvaDirective]
                }]
        }] });

/**
 * Base ControlValueAccessor control class.
 * Used in conjunction with CvaDirective.
 *
 * This class performs generic change detection based on `CvaDirective` outputs.
 */
class CvaControl {
    constructor() {
        /**
         * Control value accessor directive instance.
         */
        this.cvaDirective = inject(CvaDirective, {
            self: true,
            optional: true
        });
        /**
         * Change detector instance.
         */
        this._changeDetector = inject(ChangeDetectorRef);
        /** @Hidden */
        this._destroyRef = inject(DestroyRef);
    }
    /** @hidden */
    listenToChanges() {
        this.cvaDirective?.markForCheck.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => {
            this._changeDetector.detectChanges();
        });
        this.cvaDirective?.detectChanges.pipe(takeUntilDestroyed(this._destroyRef)).subscribe(() => {
            this._changeDetector.detectChanges();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: CvaControl, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: CvaControl }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.3", ngImport: i0, type: CvaControl, decorators: [{
            type: Injectable
        }] });

/**
 * FormField base class.
 *
 * This class is used to create form field components.
 *
 */
class FormField {
}

/** @hidden */
function isSelectableItem(item) {
    return (item &&
        item.label !== undefined &&
        item.value !== undefined &&
        Object.prototype.hasOwnProperty.call(item, 'selected'));
}
/** @hidden */
function isSelectItem(item) {
    return item && item.label !== undefined && item.value !== undefined;
}
const isOptionItem = isSelectItem;
const isSelectableOptionItem = isSelectableItem;

/**
 * Generated bundle index. Do not edit.
 */

export { CvaControl, CvaDirective, FD_FORM_FIELD, FD_FORM_FIELD_CONTROL, FormField, FormsModule, formStates, isOptionItem, isSelectItem, isSelectableItem, isSelectableOptionItem, isValidControlState };
//# sourceMappingURL=fundamental-ngx-cdk-forms.mjs.map
