import { AfterContentInit, EventEmitter, OnDestroy, QueryList } from '@angular/core';
import { Observable } from 'rxjs';
import { DndItem, ElementPosition, FdDndDropType, FdDropEvent } from '../dnd.interfaces';
import * as i0 from "@angular/core";
export type DropPredicate<T> = (dragItem: T, dropItem: T, event: FdDropEvent<T>) => boolean | Promise<boolean> | Observable<boolean>;
export type DragoverPredicate<T> = (dragItem: T, dragOverItem: T, dragItemIndex: number, dragOverItemIndex: number) => boolean;
export declare class DndListDirective<T> implements AfterContentInit, OnDestroy {
    /**
     * Defines if the element is allowed to be dragged in 2 dimensions,
     * When true - replace indicator will be displayed vertically
     */
    gridMode: boolean;
    /**
     * Defines drop strategy:
     * * `shift` mode will create line after the closest drop element.
     * * `group` mode will create replace indicator on a whole closest drop element.
     * * `auto` mode will create a line after the closest drop element
     * if dragged element coordinates are shifted for 30% from the center of the closest drop element.
     * Otherwise, it will create replace indicator on a whole closest drop element.
     *
     * `shift` mode is the default.
     */
    dropMode: FdDndDropType;
    /**
     * Threshold of dragged item over another item to define which type of `dropMode` should be used.
     */
    threshold: number;
    /** Array of items, that will be sorted */
    items: Array<T>;
    /** Defines if drag and drop feature should be enabled for list items */
    set draggable(draggable: boolean);
    /** Predicate function that checks whether the item can be dropped over another item. */
    dropPredicate: DropPredicate<T> | undefined;
    /**
     * Predicate function that checks whether the item can be dragged over another item.
     * If the function returns `false`, dragged over item will not be highlighted, and drop event will be canceled.
     */
    dragoverPredicate: DragoverPredicate<T> | undefined;
    /**
     * Event emitted when `dropPredicate` function evaluation changes.
     * Emits `true` when evaluation started, and `false` when completed.
     */
    dropPredicateCalculating: EventEmitter<boolean>;
    /** Event that is thrown when items are reordered */
    readonly itemsChange: EventEmitter<T[]>;
    /** Event that is thrown when the item is dropped */
    readonly itemDropped: EventEmitter<FdDropEvent<T>>;
    /** Event that is thrown when predicate function returned false. */
    readonly dropCancelled: EventEmitter<void>;
    /** @hidden */
    dndItems: QueryList<DndItem<T>>;
    /** @hidden */
    private readonly _initialClass;
    /** @hidden */
    private _elementsCoordinates;
    /** @hidden */
    private _closestItemIndex;
    /** @hidden */
    private _closestItemPosition;
    /** An RxJS Subject that will kill the current data stream (for unsubscribing)  */
    private readonly _refresh$;
    /** @hidden */
    private readonly _destroyRef;
    /** @hidden  */
    private _dndItemReference;
    /** @hidden */
    private _draggable;
    /** @hidden */
    private _detectedDropMode;
    /** @hidden */
    private _linesRemoved;
    /** @hidden */
    private _indicatorsRemoved;
    /** @hidden */
    private _draggedItem;
    /** @hidden */
    private _ignoreDrop;
    /** @hidden */
    ngAfterContentInit(): void;
    /** @hidden */
    ngOnDestroy(): void;
    /**
     * Refreshes the indexes of the items.
     */
    refreshQueryList(): void;
    /** Method called when the item is being moved by 1 px */
    onMove(mousePosition: ElementPosition, draggedItemIndex: number): void;
    /** Method called when an element is started to be dragged */
    dragStart(index: number): void;
    /** Method called when an element is released */
    dragEnd(draggedItemIndex: number): Promise<void>;
    /** @hidden */
    private _setDisabledItem;
    /** @hidden */
    private _selectDropModeIndicator;
    /** @hidden */
    private _removeAllLines;
    /** @hidden */
    private _removeAllReplaceIndicators;
    /** @hidden */
    private _createLine;
    /** @hidden */
    private _createReplaceIndicator;
    /**
     *  @hidden
     * Return information if an element is placed before the dragged element
     */
    private _isBefore;
    /** @hidden */
    private _changeDraggableState;
    static ɵfac: i0.ɵɵFactoryDeclaration<DndListDirective<any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<DndListDirective<any>, "[fdkDndList], [fd-dnd-list]", never, { "gridMode": { "alias": "gridMode"; "required": false; }; "dropMode": { "alias": "dropMode"; "required": false; }; "threshold": { "alias": "threshold"; "required": false; }; "items": { "alias": "items"; "required": false; }; "draggable": { "alias": "draggable"; "required": false; }; "dropPredicate": { "alias": "dropPredicate"; "required": false; }; "dragoverPredicate": { "alias": "dragoverPredicate"; "required": false; }; }, { "dropPredicateCalculating": "dropPredicateCalculating"; "itemsChange": "itemsChange"; "itemDropped": "itemDropped"; "dropCancelled": "dropCancelled"; }, ["dndItems"], never, true, never>;
}
