import { ComponentType, Overlay, OverlayRef } from '@angular/cdk/overlay';
import { EmbeddedViewRef, InjectionToken, Injector, StaticProvider, TemplateRef, Type } from '@angular/core';
import { BaseToastPosition } from './base-toast-positions';
import { BaseToastConfig } from './classes/base-toast-config';
import { BaseToastRef } from './classes/base-toast-ref';
import { ToastContainerComponent } from './interfaces/toast-container-component.interface';
import { ToastTextComponent } from './interfaces/toast-text-component.interface';
import * as i0 from "@angular/core";
export declare abstract class BaseToastService<P extends BaseToastConfig, C extends ToastContainerComponent<P> = ToastContainerComponent<P>> {
    overlay: Overlay;
    injector: Injector;
    /** Component for simple text toast. */
    protected abstract toastTextComponent: Type<ToastTextComponent>;
    /** Component for Toast Container. */
    protected abstract toastContainerComponent: Type<C>;
    /** Injection token for Toast Data. */
    protected abstract toastDataInjectionToken: InjectionToken<any>;
    /** Toast Position Strategy. */
    protected abstract toastPositionStrategy: BaseToastPosition;
    /** Toast default config. */
    protected abstract defaultConfig: P;
    /**
     * Opens a Toast with the provided configuration.
     * @param toast accepts string, Component or TemplateRef.
     */
    abstract open<T>(toast: string | Type<T> | TemplateRef<T>): BaseToastRef<ToastTextComponent | T | EmbeddedViewRef<T>, P>;
    abstract openFromString(message: string, config: P): BaseToastRef<ToastTextComponent, P>;
    /**
     * Opens a Toast with provided component inside.
     * @param component Component to render inside a Toast.
     * @param config Toast configuration.
     * @returns Toast reference.
     */
    abstract openFromComponent<T>(component: Type<T>, config: P): BaseToastRef<T, P>;
    /**
     * Opens a Toast with provided Template Reference.
     * @param template Template Reference to render inside a Toast.
     * @param config Toast configuration.
     * @returns Toast reference.
     */
    abstract openFromTemplate(template: TemplateRef<any>, config: P): BaseToastRef<EmbeddedViewRef<any>>;
    /**
     * Gets Toast Container providers.
     * @param config Toast Config.
     */
    protected abstract getContainerComponentProviders(config: P): StaticProvider[];
    /**
     * Gets Toast Content providers.
     * @param config Toast Config.
     * @param toastRef Toast Reference.
     */
    protected abstract getContentComponentProviders<T>(config: P, toastRef: BaseToastRef<T>): StaticProvider[];
    /**
     * Creates Toast Reference for provided container and overlay.
     * @param containerRef Container reference.
     * @param overlayRef Overlay Reference.
     * @param positionStrategy Position Strategy
     */
    protected abstract getToastRef<T>(containerRef: C, overlayRef: OverlayRef, positionStrategy: BaseToastPosition): BaseToastRef<T | EmbeddedViewRef<any>, P>;
    /**
     * @hidden
     * Array of all current toasts.
     */
    protected _toasts: BaseToastRef[];
    /**
     * @hidden
     */
    protected _toastsMap: Map<BaseToastPosition, BaseToastRef<any, any, ToastContainerComponent<any>>[]>;
    /** @hidden */
    private _destroyRef;
    /** @hidden */
    protected constructor(overlay: Overlay, injector: Injector);
    /**
     * Dismisses all Toasts.
     */
    hideAll(): void;
    /**
     * Returns `true` if there are some message toasts currently open. `False` otherwise.
     */
    hasOpenMessageToasts(): boolean;
    /**
     * Sets new position strategy for all Toast items.
     * @param newStrategy New Position Strategy
     */
    setNewPositionStrategy(newStrategy: BaseToastPosition): void;
    /**
     * @hidden
     * Places a new component or a template as the content of the Toast container.
     */
    protected _attach<T>(content: ComponentType<T> | TemplateRef<T>, userConfig?: P): BaseToastRef<T | EmbeddedViewRef<any>, P>;
    /**
     * Attaches the Toast container component to the overlay.
     */
    protected attachToastContainerComponent(overlayRef: OverlayRef, config: P): C;
    /**
     * Creates an injector to be used inside a Toast component.
     * @param config Config that was used to create the Toast.
     * @param toastRef Reference to the Toast.
     */
    protected createContentComponentInjector<T>(config: P, toastRef: BaseToastRef<T>): Injector;
    /**
     * Animates the old Toast out and the new one in.
     */
    protected animateToast<T>(toastRef: BaseToastRef<T | EmbeddedViewRef<any>, P>): void;
    /**
     * @hidden
     * Creates a new overlay and places it in the correct location.
     */
    private _createOverlay;
    /**
     * @hidden
     * @returns Initial Position Strategy of the Toast Overlay Reference.
     */
    private _getPositionStrategy;
    /**
     * @hidden
     * @param needlePosition
     * @private
     */
    private _getLastToastWithSamePosition;
    /**
     * @hidden
     * @param connectedElm element to connect with.
     * @param positionStrategy Position Strategy
     * @returns Flexible Connected Position Strategy for Overlay Reference.
     */
    private _composeFlexibleConnectedPosition;
    /**
     * @hidden
     * @returns Default Global Position for Overlay Reference.
     */
    private _composeGlobalPosition;
    /** @hidden */
    private _refreshOverlayPositions;
    /** @hidden */
    private _updateOverlayPositionStrategy;
    /**
     * @hidden
     * Updates the position of a current Overlay Reference.
     * @returns New Position Strategy.
     * @param toast Toast Reference
     */
    private _updatePositionStrategy;
    /** @hidden */
    private _isBoundGlobalPosition;
    static ɵfac: i0.ɵɵFactoryDeclaration<BaseToastService<any, any>, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<BaseToastService<any, any>>;
}
